@@ -16,12 +16,14 @@ package hugolib
 import (
 	"errors"
 	"fmt"
-
-	"github.com/gohugoio/hugo/hugolib/paths"
-
 	"io"
 	"strings"
 
+	"github.com/gohugoio/hugo/common/herrors"
+
+	"github.com/gohugoio/hugo/hugolib/paths"
+	_errors "github.com/pkg/errors"
+
 	"github.com/gohugoio/hugo/langs"
 
 	"github.com/gohugoio/hugo/config"
@@ -105,12 +107,23 @@ func LoadConfig(d ConfigSourceDescriptor, doWithConfig ...func(cfg config.Provid
 	v.SetConfigFile(configFilenames[0])
 	v.AddConfigPath(d.Path)
 
+	applyFileContext := func(filename string, err error) error {
+		err, _ = herrors.WithFileContextForFile(
+			err,
+			filename,
+			filename,
+			fs,
+			herrors.SimpleLineMatcher)
+
+		return err
+	}
+
 	var configFileErr error
 
 	err := v.ReadInConfig()
 	if err != nil {
 		if _, ok := err.(viper.ConfigParseError); ok {
-			return nil, configFiles, err
+			return nil, configFiles, applyFileContext(v.ConfigFileUsed(), err)
 		}
 		configFileErr = ErrNoConfigFile
 	}
@@ -128,7 +141,7 @@ func LoadConfig(d ConfigSourceDescriptor, doWithConfig ...func(cfg config.Provid
 				return nil, configFiles, fmt.Errorf("Unable to open Config file.\n (%s)\n", err)
 			}
 			if err = v.MergeConfig(r); err != nil {
-				return nil, configFiles, fmt.Errorf("Unable to parse/merge Config file (%s).\n (%s)\n", configFile, err)
+				return nil, configFiles, applyFileContext(configFile, err)
 			}
 			configFiles = append(configFiles, configFile)
 		}
@@ -205,7 +218,7 @@ func loadLanguageSettings(cfg config.Provider, oldLangs langs.Languages) error {
 	} else {
 		languages2, err = toSortedLanguages(cfg, languages)
 		if err != nil {
-			return fmt.Errorf("Failed to parse multilingual config: %s", err)
+			return _errors.Wrap(err, "Failed to parse multilingual config")
 		}
 	}
 
@@ -285,6 +298,11 @@ func loadThemeConfig(d ConfigSourceDescriptor, v1 *viper.Viper) ([]string, error
 	if err != nil {
 		return nil, err
 	}
+
+	if len(themeConfigs) == 0 {
+		return nil, nil
+	}
+
 	v1.Set("allThemes", themeConfigs)
 
 	var configFilenames []string
@@ -328,12 +346,12 @@ func applyThemeConfig(v1 *viper.Viper, theme paths.ThemeConfig) error {
 	// Only add params and new menu entries, we do not add language definitions.
 	if v1.IsSet(languagesKey) && v2.IsSet(languagesKey) {
 		v1Langs := v1.GetStringMap(languagesKey)
-		for k, _ := range v1Langs {
+		for k := range v1Langs {
 			langParamsKey := languagesKey + "." + k + "." + paramsKey
 			mergeStringMapKeepLeft(paramsKey, langParamsKey, v1, v2)
 		}
 		v2Langs := v2.GetStringMap(languagesKey)
-		for k, _ := range v2Langs {
+		for k := range v2Langs {
 			if k == "" {
 				continue
 			}
@@ -466,7 +484,7 @@ func loadDefaultSettingsFor(v *viper.Viper) error {
 	v.SetDefault("disableFastRender", false)
 	v.SetDefault("timeout", 10000) // 10 seconds
 
-	// Remove in Hugo 0.39
+	// Remove in Hugo 0.50
 
 	if v.GetBool("useModTimeAsFallback") {
 
@@ -475,7 +493,7 @@ func loadDefaultSettingsFor(v *viper.Viper) error {
 [frontmatter]
 date = [ "date",":fileModTime", ":default"]
 lastmod = ["lastmod" ,":fileModTime", ":default"]
-`, false)
+`, true)
 
 	}
 
