@@ -19,11 +19,10 @@ import (
 	"errors"
 	"fmt"
 	"reflect"
-	"unicode"
-
-	"github.com/gohugoio/hugo/media"
 
 	"github.com/gohugoio/hugo/common/maps"
+	"github.com/gohugoio/hugo/media"
+	_errors "github.com/pkg/errors"
 
 	"github.com/gohugoio/hugo/langs"
 
@@ -36,7 +35,6 @@ import (
 	"github.com/gohugoio/hugo/resource"
 
 	"github.com/gohugoio/hugo/output"
-	"github.com/gohugoio/hugo/parser"
 	"github.com/mitchellh/mapstructure"
 
 	"html/template"
@@ -50,7 +48,6 @@ import (
 	"time"
 	"unicode/utf8"
 
-	bp "github.com/gohugoio/hugo/bufferpool"
 	"github.com/gohugoio/hugo/compare"
 	"github.com/gohugoio/hugo/source"
 	"github.com/spf13/cast"
@@ -140,6 +137,7 @@ type Page struct {
 	contentv        template.HTML
 	summary         template.HTML
 	TableOfContents template.HTML
+
 	// Passed to the shortcodes
 	pageWithoutContent *PageWithoutContent
 
@@ -160,7 +158,6 @@ type Page struct {
 
 	extension   string
 	contentType string
-	renderable  bool
 
 	Layout string
 
@@ -170,19 +167,12 @@ type Page struct {
 
 	linkTitle string
 
-	frontmatter []byte
-
-	// rawContent is the raw content read from the content file.
-	rawContent []byte
-
-	// workContent is a copy of rawContent that may be mutated during site build.
-	workContent []byte
+	// Content items.
+	pageContent
 
 	// whether the content is in a CJK language.
 	isCJKLanguage bool
 
-	shortcodeState *shortcodeHandler
-
 	// the content stripped for HTML
 	plain      string // TODO should be []byte
 	plainWords []string
@@ -193,7 +183,7 @@ type Page struct {
 	// menus
 	pageMenus PageMenus
 
-	Source
+	source.File
 
 	Position `json:"-"`
 
@@ -307,13 +297,13 @@ func (p *Page) initContent() {
 
 			err = p.prepareForRender()
 			if err != nil {
-				p.s.Log.ERROR.Printf("Failed to prepare page %q for render: %s", p.Path(), err)
+				c <- err
 				return
 			}
 
 			if len(p.summary) == 0 {
 				if err = p.setAutoSummary(); err != nil {
-					err = fmt.Errorf("Failed to set user auto summary for page %q: %s", p.pathOrTitle(), err)
+					err = p.errorf(err, "failed to set auto summary")
 				}
 			}
 			c <- err
@@ -324,7 +314,7 @@ func (p *Page) initContent() {
 			p.s.Log.WARN.Printf("WARNING: Timed out creating content for page %q (.Content will be empty). This is most likely a circular shortcode content loop that should be fixed. If this is just a shortcode calling a slow remote service, try to set \"timeout=20000\" (or higher, value is in milliseconds) in config.toml.\n", p.pathOrTitle())
 		case err := <-c:
 			if err != nil {
-				p.s.Log.ERROR.Println(err)
+				p.s.SendError(err)
 			}
 		}
 	})
@@ -475,7 +465,7 @@ func (p *Page) BundleType() string {
 		return "branch"
 	}
 
-	var source interface{} = p.Source.File
+	var source interface{} = p.File
 	if fi, ok := source.(*fileInfo); ok {
 		switch fi.bundleTp {
 		case bundleBranch:
@@ -492,11 +482,6 @@ func (p *Page) MediaType() media.Type {
 	return media.OctetType
 }
 
-type Source struct {
-	Frontmatter []byte
-	Content     []byte
-	source.File
-}
 type PageMeta struct {
 	wordCount      int
 	fuzzyWordCount int
@@ -505,8 +490,8 @@ type PageMeta struct {
 }
 
 type Position struct {
-	Prev          *Page
-	Next          *Page
+	PrevPage      *Page
+	NextPage      *Page
 	PrevInSection *Page
 	NextInSection *Page
 }
@@ -519,7 +504,7 @@ func (ps Pages) String() string {
 
 func (ps Pages) findPagePosByFilename(filename string) int {
 	for i, x := range ps {
-		if x.Source.Filename() == filename {
+		if x.Filename() == filename {
 			return i
 		}
 	}
@@ -552,8 +537,8 @@ func (ps Pages) findPagePosByFilnamePrefix(prefix string) int {
 
 	// Find the closest match
 	for i, x := range ps {
-		if strings.HasPrefix(x.Source.Filename(), prefix) {
-			diff := len(x.Source.Filename()) - prefixLen
+		if strings.HasPrefix(x.Filename(), prefix) {
+			diff := len(x.Filename()) - prefixLen
 			if lenDiff == -1 || diff < lenDiff {
 				lenDiff = diff
 				currPos = i
@@ -567,18 +552,13 @@ func (ps Pages) findPagePosByFilnamePrefix(prefix string) int {
 // will return -1 if not found
 func (ps Pages) findPagePos(page *Page) int {
 	for i, x := range ps {
-		if x.Source.Filename() == page.Source.Filename() {
+		if x.Filename() == page.Filename() {
 			return i
 		}
 	}
 	return -1
 }
 
-func (p *Page) createWorkContentCopy() {
-	p.workContent = make([]byte, len(p.rawContent))
-	copy(p.workContent, p.rawContent)
-}
-
 func (p *Page) Plain() string {
 	p.initContent()
 	p.initPlain(true)
@@ -708,59 +688,11 @@ func (p *Page) Authors() AuthorList {
 }
 
 func (p *Page) UniqueID() string {
-	return p.Source.UniqueID()
+	return p.File.UniqueID()
 }
 
-// for logging
-func (p *Page) lineNumRawContentStart() int {
-	return bytes.Count(p.frontmatter, []byte("\n")) + 1
-}
-
-var (
-	internalSummaryDivider = []byte("HUGOMORE42")
-)
-
-// replaceDivider replaces the <!--more--> with an internal value and returns
-// whether the contentis truncated or not.
-// Note: The content slice will be modified if needed.
-func replaceDivider(content, from, to []byte) ([]byte, bool) {
-	dividerIdx := bytes.Index(content, from)
-	if dividerIdx == -1 {
-		return content, false
-	}
-
-	afterSummary := content[dividerIdx+len(from):]
-
-	// If the raw content has nothing but whitespace after the summary
-	// marker then the page shouldn't be marked as truncated.  This check
-	// is simplest against the raw content because different markup engines
-	// (rst and asciidoc in particular) add div and p elements after the
-	// summary marker.
-	truncated := bytes.IndexFunc(afterSummary, func(r rune) bool { return !unicode.IsSpace(r) }) != -1
-
-	content = append(content[:dividerIdx], append(to, afterSummary...)...)
-
-	return content, truncated
-
-}
-
-// We have to replace the <!--more--> with something that survives all the
-// rendering engines.
-func (p *Page) replaceDivider(content []byte) []byte {
-	summaryDivider := helpers.SummaryDivider
-	if p.Markup == "org" {
-		summaryDivider = []byte("# more")
-	}
-
-	replaced, truncated := replaceDivider(content, summaryDivider, internalSummaryDivider)
-
-	p.truncated = truncated
-
-	return replaced
-}
-
-// Returns the page as summary and main if a user defined split is provided.
-func (p *Page) setUserDefinedSummaryIfProvided(rawContentCopy []byte) (*summaryContent, error) {
+// Returns the page as summary and main.
+func (p *Page) setUserDefinedSummary(rawContentCopy []byte) (*summaryContent, error) {
 
 	sc, err := splitUserDefinedSummaryAndContent(p.Markup, rawContentCopy)
 
@@ -930,7 +862,7 @@ func (s *Site) newPageFromFile(fi *fileInfo) *Page {
 		pageContentInit: &pageContentInit{},
 		Kind:            kindFromFileInfo(fi),
 		contentType:     "",
-		Source:          Source{File: fi},
+		File:            fi,
 		Keywords:        []string{}, Sitemap: Sitemap{Priority: -1},
 		params:       make(map[string]interface{}),
 		translations: make(Pages, 0),
@@ -963,15 +895,18 @@ func (p *Page) Section() string {
 	if p.Kind == KindSection || p.Kind == KindTaxonomy || p.Kind == KindTaxonomyTerm {
 		return p.sections[0]
 	}
-	return p.Source.Section()
+	return p.File.Section()
 }
 
-func (s *Site) NewPageFrom(buf io.Reader, name string) (*Page, error) {
+func (s *Site) newPageFrom(buf io.Reader, name string) (*Page, error) {
 	p, err := s.NewPage(name)
 	if err != nil {
 		return p, err
 	}
 	_, err = p.ReadFrom(buf)
+	if err != nil {
+		return nil, err
+	}
 
 	return p, err
 }
@@ -992,11 +927,15 @@ func (s *Site) NewPage(name string) (*Page, error) {
 func (p *Page) ReadFrom(buf io.Reader) (int64, error) {
 	// Parse for metadata & body
 	if err := p.parse(buf); err != nil {
-		p.s.Log.ERROR.Printf("%s for %s", err, p.File.Path())
-		return 0, err
+		return 0, p.errWithFileContext(err)
+
 	}
 
-	return int64(len(p.rawContent)), nil
+	if err := p.mapContent(); err != nil {
+		return 0, p.errWithFileContext(err)
+	}
+
+	return int64(len(p.source.parsed.Input())), nil
 }
 
 func (p *Page) WordCount() int {
@@ -1205,7 +1144,7 @@ func (p *Page) initMainOutputFormat() error {
 	pageOutput, err := newPageOutput(p, false, false, outFormat)
 
 	if err != nil {
-		return fmt.Errorf("Failed to create output page for type %q for page %q: %s", outFormat.Name, p.pathOrTitle(), err)
+		return p.errorf(err, "failed to create output page for type %q", outFormat.Name)
 	}
 
 	p.mainPageOutput = pageOutput
@@ -1271,13 +1210,13 @@ func (p *Page) prepareForRender() error {
 	// Note: The shortcodes in a page cannot access the page content it lives in,
 	// hence the withoutContent().
 	if workContentCopy, err = handleShortcodes(p.withoutContent(), workContentCopy); err != nil {
-		s.Log.ERROR.Printf("Failed to handle shortcodes for page %s: %s", p.BaseFileName(), err)
+		return err
 	}
 
-	if p.Markup != "html" {
+	if p.Markup != "html" && p.source.hasSummaryDivider {
 
 		// Now we know enough to create a summary of the page and count some words
-		summaryContent, err := p.setUserDefinedSummaryIfProvided(workContentCopy)
+		summaryContent, err := p.setUserDefinedSummary(workContentCopy)
 
 		if err != nil {
 			s.Log.ERROR.Printf("Failed to set user defined summary for page %q: %s", p.Path(), err)
@@ -1294,9 +1233,7 @@ func (p *Page) prepareForRender() error {
 	return nil
 }
 
-var ErrHasDraftAndPublished = errors.New("both draft and published parameters were found in page's frontmatter")
-
-func (p *Page) update(frontmatter map[string]interface{}) error {
+func (p *Page) updateMetaData(frontmatter map[string]interface{}) error {
 	if frontmatter == nil {
 		return errors.New("missing frontmatter data")
 	}
@@ -1304,8 +1241,8 @@ func (p *Page) update(frontmatter map[string]interface{}) error {
 	maps.ToLower(frontmatter)
 
 	var mtime time.Time
-	if p.Source.FileInfo() != nil {
-		mtime = p.Source.FileInfo().ModTime()
+	if p.FileInfo() != nil {
+		mtime = p.FileInfo().ModTime()
 	}
 
 	var gitAuthorDate time.Time
@@ -1318,7 +1255,7 @@ func (p *Page) update(frontmatter map[string]interface{}) error {
 		Params:        p.params,
 		Dates:         &p.PageDates,
 		PageURLs:      &p.URLPath,
-		BaseFilename:  p.BaseFileName(),
+		BaseFilename:  p.ContentBaseName(),
 		ModTime:       mtime,
 		GitAuthorDate: gitAuthorDate,
 	}
@@ -1440,9 +1377,7 @@ func (p *Page) update(frontmatter map[string]interface{}) error {
 					resources = append(resources, cast.ToStringMap(vvv))
 				}
 			case []map[string]interface{}:
-				for _, vvv := range vv {
-					resources = append(resources, vvv)
-				}
+				resources = append(resources, vv...)
 			case []interface{}:
 				for _, vvv := range vv {
 					switch vvvv := vvv.(type) {
@@ -1509,13 +1444,12 @@ func (p *Page) update(frontmatter map[string]interface{}) error {
 	p.Markup = helpers.GuessType(p.Markup)
 	if p.Markup == "unknown" {
 		// Fall back to file extension (might also return "unknown")
-		p.Markup = helpers.GuessType(p.Source.Ext())
+		p.Markup = helpers.GuessType(p.Ext())
 	}
 
 	if draft != nil && published != nil {
 		p.Draft = *draft
-		p.s.Log.ERROR.Printf("page %s has both draft and published settings in its frontmatter. Using draft.", p.File.Path())
-		return ErrHasDraftAndPublished
+		p.s.Log.WARN.Printf("page %q has both draft and published settings in its frontmatter. Using draft.", p.File.Path())
 	} else if draft != nil {
 		p.Draft = *draft
 	} else if published != nil {
@@ -1526,7 +1460,7 @@ func (p *Page) update(frontmatter map[string]interface{}) error {
 	if isCJKLanguage != nil {
 		p.isCJKLanguage = *isCJKLanguage
 	} else if p.s.Cfg.GetBool("hasCJKLanguage") {
-		if cjk.Match(p.rawContent) {
+		if cjk.Match(p.source.parsed.Input()) {
 			p.isCJKLanguage = true
 		} else {
 			p.isCJKLanguage = false
@@ -1751,128 +1685,13 @@ func (p *Page) shouldRenderTo(f output.Format) bool {
 	return found
 }
 
-func (p *Page) parse(reader io.Reader) error {
-	psr, err := parser.ReadFrom(reader)
-	if err != nil {
-		return err
-	}
-
-	p.renderable = psr.IsRenderable()
-	p.frontmatter = psr.FrontMatter()
-	p.rawContent = psr.Content()
-	p.lang = p.Source.File.Lang()
-
-	meta, err := psr.Metadata()
-	if err != nil {
-		return fmt.Errorf("failed to parse page metadata for %q: %s", p.File.Path(), err)
-	}
-	if meta == nil {
-		// missing frontmatter equivalent to empty frontmatter
-		meta = map[string]interface{}{}
-	}
-
-	if p.s != nil && p.s.owner != nil {
-		gi, enabled := p.s.owner.gitInfo.forPage(p)
-		if gi != nil {
-			p.GitInfo = gi
-		} else if enabled {
-			p.s.Log.WARN.Printf("Failed to find GitInfo for page %q", p.Path())
-		}
-	}
-
-	return p.update(meta)
-}
-
+// RawContent returns the un-rendered source content without
+// any leading front matter.
 func (p *Page) RawContent() string {
-	return string(p.rawContent)
-}
-
-func (p *Page) SetSourceContent(content []byte) {
-	p.Source.Content = content
-}
-
-func (p *Page) SetSourceMetaData(in interface{}, mark rune) (err error) {
-	// See https://github.com/gohugoio/hugo/issues/2458
-	defer func() {
-		if r := recover(); r != nil {
-			var ok bool
-			err, ok = r.(error)
-			if !ok {
-				err = fmt.Errorf("error from marshal: %v", r)
-			}
-		}
-	}()
-
-	buf := bp.GetBuffer()
-	defer bp.PutBuffer(buf)
-
-	err = parser.InterfaceToFrontMatter(in, mark, buf)
-	if err != nil {
-		return
+	if p.source.posMainContent == -1 {
+		return ""
 	}
-
-	_, err = buf.WriteRune('\n')
-	if err != nil {
-		return
-	}
-
-	p.Source.Frontmatter = buf.Bytes()
-
-	return
-}
-
-func (p *Page) SafeSaveSourceAs(path string) error {
-	return p.saveSourceAs(path, true)
-}
-
-func (p *Page) SaveSourceAs(path string) error {
-	return p.saveSourceAs(path, false)
-}
-
-func (p *Page) saveSourceAs(path string, safe bool) error {
-	b := bp.GetBuffer()
-	defer bp.PutBuffer(b)
-
-	b.Write(p.Source.Frontmatter)
-	b.Write(p.Source.Content)
-
-	bc := make([]byte, b.Len(), b.Len())
-	copy(bc, b.Bytes())
-
-	return p.saveSource(bc, path, safe)
-}
-
-func (p *Page) saveSource(by []byte, inpath string, safe bool) (err error) {
-	if !filepath.IsAbs(inpath) {
-		inpath = p.s.PathSpec.AbsPathify(inpath)
-	}
-	p.s.Log.INFO.Println("creating", inpath)
-	if safe {
-		err = helpers.SafeWriteToDisk(inpath, bytes.NewReader(by), p.s.Fs.Source)
-	} else {
-		err = helpers.WriteToDisk(inpath, bytes.NewReader(by), p.s.Fs.Source)
-	}
-	if err != nil {
-		return
-	}
-	return nil
-}
-
-func (p *Page) SaveSource() error {
-	return p.SaveSourceAs(p.FullFilePath())
-}
-
-// TODO(bep) lazy consolidate
-func (p *Page) processShortcodes() error {
-	p.shortcodeState = newShortcodeHandler(p)
-	tmpContent, err := p.shortcodeState.extractShortcodes(string(p.workContent), p.withoutContent())
-	if err != nil {
-		return err
-	}
-	p.workContent = []byte(tmpContent)
-
-	return nil
-
+	return string(p.source.parsed.Input()[p.source.posMainContent:])
 }
 
 func (p *Page) FullFilePath() string {
@@ -1888,8 +1707,8 @@ func (p *Page) FullFilePath() string {
 // For pages that do not (sections witout content page etc.), it returns the
 // virtual path, consistent with where you would add a source file.
 func (p *Page) absoluteSourceRef() string {
-	if p.Source.File != nil {
-		sourcePath := p.Source.Path()
+	if p.File != nil {
+		sourcePath := p.Path()
 		if sourcePath != "" {
 			return "/" + filepath.ToSlash(sourcePath)
 		}
@@ -2081,7 +1900,7 @@ func (p *Page) decodeRefArgs(args map[string]interface{}) (refArgs, *SiteInfo, e
 func (p *Page) Ref(argsm map[string]interface{}) (string, error) {
 	args, s, err := p.decodeRefArgs(argsm)
 	if err != nil {
-		return "", fmt.Errorf("invalid arguments to Ref: %s", err)
+		return "", _errors.Wrap(err, "invalid arguments to Ref")
 	}
 
 	if s == nil {
@@ -2101,7 +1920,7 @@ func (p *Page) Ref(argsm map[string]interface{}) (string, error) {
 func (p *Page) RelRef(argsm map[string]interface{}) (string, error) {
 	args, s, err := p.decodeRefArgs(argsm)
 	if err != nil {
-		return "", fmt.Errorf("invalid arguments to Ref: %s", err)
+		return "", _errors.Wrap(err, "invalid arguments to Ref")
 	}
 
 	if s == nil {
@@ -2305,8 +2124,20 @@ func (p *Page) setValuesForKind(s *Site) {
 
 // Used in error logs.
 func (p *Page) pathOrTitle() string {
-	if p.Path() != "" {
-		return p.Path()
+	if p.Filename() != "" {
+		return p.Filename()
 	}
 	return p.title
 }
+
+func (p *Page) Next() *Page {
+	// TODO Remove the deprecation notice (but keep PrevPage as an alias) Hugo 0.52
+	helpers.Deprecated("Page", ".Next", "Use .PrevPage (yes, not .NextPage).", false)
+	return p.PrevPage
+}
+
+func (p *Page) Prev() *Page {
+	// TODO Remove the deprecation notice (but keep NextPage as an alias) Hugo 0.52
+	helpers.Deprecated("Page", ".Prev", "Use .NextPage (yes, not .PrevPage).", false)
+	return p.NextPage
+}
