@@ -12,6 +12,7 @@ import (
 	"path/filepath"
 	"runtime"
 	"strings"
+	"sync"
 	"time"
 
 	"github.com/magefile/mage/mg"
@@ -38,24 +39,6 @@ func init() {
 	os.Setenv("GO111MODULE", "on")
 }
 
-func getDep() error {
-	if !isWindows() {
-		// We only need this on Appveyor.
-		// TODO(bep) go11 clean up the vendor stuff once Appveyor gets Go 1.11.
-		return nil
-	}
-	return sh.Run(goexe, "get", "-u", "github.com/golang/dep/cmd/dep")
-}
-
-// Install Go Dep and sync Hugo's vendored dependencies
-func Vendor() error {
-	if !isWindows() {
-		return nil
-	}
-	mg.Deps(getDep)
-	return sh.Run("dep", "ensure")
-}
-
 // Build hugo binary
 func Hugo() error {
 	return sh.RunWith(flagEnv(), goexe, "build", "-ldflags", ldflags, "-tags", buildTags(), packageName)
@@ -116,7 +99,11 @@ func Check() {
 		fmt.Printf("Skip Check on %s\n", runtime.Version())
 		return
 	}
-	mg.Deps(Test386, Fmt, Vet)
+
+	mg.Deps(Test386)
+
+	mg.Deps(Fmt, Vet)
+
 	// don't run two tests in parallel, they saturate the CPUs anyway, and running two
 	// causes memory issues in CI.
 	mg.Deps(TestRace)
@@ -179,19 +166,26 @@ func Fmt() error {
 	return nil
 }
 
-var pkgPrefixLen = len("github.com/gohugoio/hugo")
+var (
+	pkgPrefixLen = len("github.com/gohugoio/hugo")
+	pkgs         []string
+	pkgsInit     sync.Once
+)
 
 func hugoPackages() ([]string, error) {
-	mg.Deps(getDep)
-	s, err := sh.Output(goexe, "list", "./...")
-	if err != nil {
-		return nil, err
-	}
-	pkgs := strings.Split(s, "\n")
-	for i := range pkgs {
-		pkgs[i] = "." + pkgs[i][pkgPrefixLen:]
-	}
-	return pkgs, nil
+	var err error
+	pkgsInit.Do(func() {
+		var s string
+		s, err = sh.Output(goexe, "list", "./...")
+		if err != nil {
+			return
+		}
+		pkgs = strings.Split(s, "\n")
+		for i := range pkgs {
+			pkgs[i] = "." + pkgs[i][pkgPrefixLen:]
+		}
+	})
+	return pkgs, err
 }
 
 // Run golint linter
@@ -217,16 +211,14 @@ func Lint() error {
 
 //  Run go vet linter
 func Vet() error {
-	mg.Deps(getDep)
 	if err := sh.Run(goexe, "vet", "./..."); err != nil {
-		return fmt.Errorf("error running govendor: %v", err)
+		return fmt.Errorf("error running go vet: %v", err)
 	}
 	return nil
 }
 
 // Generate test coverage report
 func TestCoverHTML() error {
-	mg.Deps(getDep)
 	const (
 		coverAll = "coverage-all.out"
 		cover    = "coverage.out"
@@ -266,24 +258,10 @@ func TestCoverHTML() error {
 	return sh.Run(goexe, "tool", "cover", "-html="+coverAll)
 }
 
-// Verify that vendored packages match git HEAD
-func CheckVendor() error {
-	if err := sh.Run("git", "diff-index", "--quiet", "HEAD", "vendor/"); err != nil {
-		// yes, ignore errors from this, not much we can do.
-		sh.Exec(nil, os.Stdout, os.Stderr, "git", "diff", "vendor/")
-		return errors.New("check-vendor target failed: vendored packages out of sync")
-	}
-	return nil
-}
-
 func isGoLatest() bool {
 	return strings.Contains(runtime.Version(), "1.11")
 }
 
-func isWindows() bool {
-	return runtime.GOOS == "windows"
-}
-
 func buildTags() string {
 	// To build the extended Hugo SCSS/SASS enabled version, build with
 	// HUGO_BUILD_TAGS=extended mage install etc.
