@@ -23,7 +23,7 @@ import (
 	"time"
 
 	"github.com/gohugoio/hugo/deps"
-	jww "github.com/spf13/jwalterweatherman"
+	_errors "github.com/pkg/errors"
 )
 
 // New returns a new instance of the data-namespaced template functions.
@@ -50,7 +50,7 @@ func (ns *Namespace) GetCSV(sep string, urlParts ...string) (d [][]string, err e
 	url := strings.Join(urlParts, "")
 
 	var clearCacheSleep = func(i int, u string) {
-		jww.ERROR.Printf("Retry #%d for %s and sleeping for %s", i, url, resSleep)
+		ns.deps.Log.WARN.Printf("Retry #%d for %s and sleeping for %s", i, url, resSleep)
 		time.Sleep(resSleep)
 		deleteCache(url, ns.deps.Fs.Source, ns.deps.Cfg)
 	}
@@ -59,8 +59,7 @@ func (ns *Namespace) GetCSV(sep string, urlParts ...string) (d [][]string, err e
 		var req *http.Request
 		req, err = http.NewRequest("GET", url, nil)
 		if err != nil {
-			jww.ERROR.Printf("Failed to create request for getCSV: %s", err)
-			return nil, err
+			return nil, _errors.Wrapf(err, "failed to create request for getCSV for resource %s", url)
 		}
 
 		req.Header.Add("Accept", "text/csv")
@@ -69,22 +68,22 @@ func (ns *Namespace) GetCSV(sep string, urlParts ...string) (d [][]string, err e
 		var c []byte
 		c, err = ns.getResource(req)
 		if err != nil {
-			jww.ERROR.Printf("Failed to read csv resource %q with error message %s", url, err)
-			return nil, err
+			return nil, _errors.Wrapf(err, "failed to read CSV resource %q", url)
 		}
 
 		if !bytes.Contains(c, []byte(sep)) {
-			err = errors.New("Cannot find separator " + sep + " in CSV.")
-			return
+			return nil, _errors.Errorf("cannot find separator %s in CSV for %s", sep, url)
 		}
 
 		if d, err = parseCSV(c, sep); err != nil {
-			jww.ERROR.Printf("Failed to parse csv file %s with error message %s", url, err)
+			err = _errors.Wrapf(err, "failed to parse CSV file %s", url)
+
 			clearCacheSleep(i, url)
 			continue
 		}
 		break
 	}
+
 	return
 }
 
@@ -98,8 +97,7 @@ func (ns *Namespace) GetJSON(urlParts ...string) (v interface{}, err error) {
 		var req *http.Request
 		req, err = http.NewRequest("GET", url, nil)
 		if err != nil {
-			jww.ERROR.Printf("Failed to create request for getJSON: %s", err)
-			return nil, err
+			return nil, _errors.Wrapf(err, "Failed to create request for getJSON resource %s", url)
 		}
 
 		req.Header.Add("Accept", "application/json")
@@ -107,20 +105,22 @@ func (ns *Namespace) GetJSON(urlParts ...string) (v interface{}, err error) {
 		var c []byte
 		c, err = ns.getResource(req)
 		if err != nil {
-			jww.ERROR.Printf("Failed to get json resource %s with error message %s", url, err)
-			return nil, err
+			return nil, _errors.Wrapf(err, "failed to get getJSON resource %q", url)
 		}
-
 		err = json.Unmarshal(c, &v)
 		if err != nil {
-			jww.ERROR.Printf("Cannot read json from resource %s with error message %s", url, err)
-			jww.ERROR.Printf("Retry #%d for %s and sleeping for %s", i, url, resSleep)
+			ns.deps.Log.WARN.Printf("Cannot read JSON from resource %s: %s", url, err)
+			ns.deps.Log.WARN.Printf("Retry #%d for %s and sleeping for %s", i, url, resSleep)
 			time.Sleep(resSleep)
 			deleteCache(url, ns.deps.Fs.Source, ns.deps.Cfg)
 			continue
 		}
 		break
 	}
+
+	if err != nil {
+		return nil, _errors.Wrapf(err, "failed to get getJSON resource %q", url)
+	}
 	return
 }
 
